<?php

namespace Drupal\instagram_media\Plugin\Block;

use Drupal;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Template\Attribute;
use Drupal\Core\TempStore\PrivateTempStore;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\instagram_media\Service\InstagramMediaService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides Instagram Media block.
 *
 * @Block(
 *   id = "instagram_media",
 *   admin_label = @Translation("Instagram Media Block"),
 * )
 */
class InstagramMedia extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected AccountProxyInterface $currentUser;

  /**
   * The tempstore service.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStore
   */
  protected PrivateTempStore $tempStore;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * Configuration.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The file system service.Use different styles for posts
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Instagram media service.
   *
   * @var \Drupal\instagram_media\Service\InstagramMediaService
   */
  private InstagramMediaService $instagramMediaService;

  /**
   * Constructs a new Instagram Media Block.
   *
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Database\Connection $database
   * The database connection.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The temporary storage factory.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   *   The configuration.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\instagram_media\Service\InstagramMediaService $instagram_media_service
   *   The instagram media API service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    Connection $database,
    AccountProxyInterface $currentUser,
    PrivateTempStoreFactory $temp_store_factory,
    LanguageManagerInterface $language_manager,
    ConfigFactoryInterface $config,
    FileSystemInterface $file_system,
    ModuleHandlerInterface $module_handler,
    EntityTypeManagerInterface $entity_type_manager,
    InstagramMediaService $instagram_media_service,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->database = $database;
    $this->currentUser = $currentUser;
    $this->tempStore = $temp_store_factory->get('instagram_media');
    $this->languageManager = $language_manager;
    $this->configFactory = $config;
    $this->fileSystem = $file_system;
    $this->moduleHandler = $module_handler;
    $this->entityTypeManager = $entity_type_manager;
    $this->instagramMediaService = $instagram_media_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('database'),
      $container->get('current_user'),
      $container->get('tempstore.private'),
      $container->get('language_manager'),
      $container->get('config.factory'),
      $container->get('file_system'),
      $container->get('module_handler'),
      $container->get('entity_type.manager'),
      $container->get('instagram_media.instagram_media_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'token' => '',
      'image_style_options' => 'image_style',
      'image_style' => 'large',
      'responsive_image_style' => 'wide',
      'limit' => 6,
      'different_image_style' => 0,
      'different_responsive_style' => 0,
      'autoplay' => TRUE,
      'video' => FALSE,
      'post_caption' => FALSE,
      'media' => '',
      'header_mode' => 'tag_and_link',
      'insights' => FALSE,
      'links' => FALSE,
      'swiper' => FALSE,
      'swiper_arrow' => FALSE,
      'swiper_pagination' => FALSE,
      'fancybox' => FALSE,
      'disabled_style' => FALSE,
      'grid' => 'no-grid',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $config = $this->getConfiguration();
    $block_id = $config['block_id'];
    $style_access = $this->moduleHandler->moduleExists('image');
    $responsive_style_access = $this->moduleHandler->moduleExists('responsive_image');

    $build = [
      '#theme' => 'instagram_media_block',
      '#autoplay' => $config['autoplay'] ?: FALSE,
      '#video' => $config['video'] ?: FALSE,
      '#post_caption' => $config['post_caption'] ?: FALSE,
      '#insights' => $config['insights'] ?: FALSE,
      '#grid' => $config['grid'] ?: 'no-grid',
      '#swiper_config' => [
        'swiper' => $config['swiper'] ?: FALSE,
        'swiper_arrow' => $config['swiper_arrow'] ?: FALSE,
        'swiper_pagination' => $config['swiper_pagination'] ?: FALSE,
      ],
      '#header_mode' => $config['header_mode'] ?: 'tag_and_link',
      '#fancybox' => $config['fancybox'] ?: FALSE,
    ];

    $posts = $this->getPosts($block_id);
    $data = [];

    foreach ($posts as $key => $post) {
      $post['attributes'] = new Attribute([
        'class' => [],
        'alt' => $post['caption'] ?? '',
      ]);
      $media_uri = $post['media_url'];

      $type = $post['media_type'];

      if (!empty($config['fancybox'])) {
        $post['fancybox_full_uri'] = Drupal::service('file_url_generator')
          ->generateString($media_uri);
      }
      if ($type === 'VIDEO' && $config['video']) {
        continue;
      }

      if ($type === 'VIDEO') {
        $post['video_class'][] = 'insta-video';
        $post['media_url'] = Drupal::service('file_url_generator')
          ->generateString($media_uri);
      }
      else {
        // Processing image
        $entity = [
          '#theme' => 'image',
          '#uri' => $media_uri,
          '#attributes' => $post['attributes'],
        ];
        if ($config['image_style_options'] === 'image_style') {
          if (!empty($config['image_style']) && $style_access) {
            $entity = [
              '#theme' => 'image_style',
              '#style_name' => $config['image_style'],
              '#uri' => $media_uri,
              '#attributes' => $post['attributes'],
            ];
          }

          if ($this->configuration['different_image_style'] && !empty($this->configuration['image_styles_rendering']["image_style_$key"])) {
            $entity = [
              '#theme' => 'image_style',
              '#style_name' => $this->configuration['image_styles_rendering']["image_style_$key"],
              '#uri' => $media_uri,
              '#attributes' => $post['attributes'],
            ];
          }
        }
        elseif ($config['image_style_options'] === 'responsive_image_style') {
          if (!empty($config['responsive_image_style']) && $responsive_style_access) {
            $entity = [
              '#theme' => 'responsive_image',
              '#uri' => $media_uri,
              '#responsive_image_style_id' => $config['responsive_image_style'],
              '#attributes' => $post['attributes'],
            ];
          }
          if ($this->configuration['different_responsive_style'] && !empty($this->configuration['responsive_image_rendering']["responsive_style_$key"])) {
            $entity = [
              '#theme' => 'responsive_image',
              '#uri' => $media_uri,
              '#responsive_image_style_id' => $this->configuration['responsive_image_rendering']["responsive_style_$key"],
              '#attributes' => $post['attributes'],
            ];
          }
        }
        $post['media_url'] = $entity;

        // Attributes
        $post['attributes']['class'][] = 'insta-image';
        $post['attributes']['loading'] = 'lazy';
      }

      $data[] = $post;
    }

    $this->saveBlockData($config, $data);

    if ($config['links']) {
      $build['#user_links'] = $this->getUserLinks($block_id)[0];
    }
    if (empty($config['disabled_style'])) {
      $build['#attached']['library'][] = 'instagram_media/instagram';
    }

    $build['#posts'] = $data;
    $this->attachLibraries($build, $config);
    Drupal::moduleHandler()->alter('build', $build);

    return $build;
  }

  /**
   * Fetch Instagram posts from the database.
   */
  private function getPosts($block_id): array {
    $query = $this->database->select('instagram_media_posts', 'p')
      ->fields('p')
      ->condition('block_id',
        $block_id)
      ->execute();
    $posts = [];
    foreach ($query as $post) {
      $posts[] = [
        'media_type' => $post->media_type,
        'media_url' => $post->media_url,
        'caption' => $post->caption,
        'like_count' => $post->like_count,
        'comments_count' => $post->comments_count,
        'timestamp' => date('Y-m-d H:i:s', $post->timestamp),
        'permalink' => $post->permalink,
      ];
    }
    return $posts;
  }

  /**
   * Save processed media data to block configuration.
   */
  protected function saveBlockData($config, $data): void {
    if (isset($config['block_id'])) {
      Drupal::state()->set('instagram_media_posts', $data);
    }
  }

  /**
   * Retrieve Instagram user links.
   */
  protected function getUserLinks($block_id): array {
    $query_links = $this->database->select('instagram_media_links', 'l')
      ->fields('l')
      ->condition('block_id', $block_id)
      ->execute();

    $links = [];
    foreach ($query_links as $link) {
      $links[] = [
        'name' => $link->name,
        'username' => $link->username,
        'profile_url' => $link->profile_url,
        'profile_picture_url' => $link->profile_picture_url,
        'media_count' => $link->media_count,
        'followers_count' => $link->followers_count,
        'follows_count' => $link->follows_count,
      ];
    }
    return $links;
  }

  /**
   * Attach necessary libraries for rendering.
   */
  private function attachLibraries(&$build, $config): void {
    if (!empty($config['disabled_style'])) {
      $build['#attached']['library'][] = 'instagram_media/instagram';
    }
    if (!empty($config['fancybox'])) {
      $active_theme = Drupal::service('theme.manager')
        ->getActiveTheme()
        ->getName();
      $build['#attached']['library'][] = "$active_theme/fancybox";
    }
    if (!empty($config['swiper'])) {
      $active_theme = Drupal::service('theme.manager')
        ->getActiveTheme()
        ->getName();
      $build['#attached']['library'][] = "$active_theme/swiper";
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags(): array {
    return Cache::mergeTags(
      parent::getCacheTags(), ['instagram_media:media']
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form['#prefix'] = '<div id="instagram-form--settings">';
    $form['#suffix'] = '</div>';
    $config = $this->getConfiguration();

    $responsive_style_access = $this->moduleHandler->moduleExists('responsive_image');

    $form['token'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Instagram Token'),
      '#rows' => 3,
      '#cols' => 5,
      '#required' => TRUE,
      '#default_value' => $config['token'] ?: '',
      '#description' => $this->t('Paste long lived access token.'),
      '#description_display' => 'after',
    ];

    //    Image and Video settings
    $form['media_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Media settings'),
      '#attributes' => [
        'open' => TRUE,
      ],
    ];

    $form['media_settings']['image_style_options'] = [
      '#type' => 'select',
      '#title' => $this->t('Image style options'),
      '#options' => [
        '' => t('- None -'),
        'image_style' => $this->t('Image style'),
        'responsive_image_style' => $this->t('Responsive image style'),
      ],
      '#default_value' => $config['image_style_options'] ?: '',
      '#attributes' => ['class' => ['image_style_options']],
      '#ajax' => [
        'callback' => [$this, 'formSettingsAjax'],
        'event' => 'change',
        'wrapper' => 'instagram-form--settings',
        'progress' => ['type' => 'throbber'],
      ],
    ];

    $form['media_settings']['image_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Image style'),
      '#options' => image_style_options(),
      '#default_value' => $config['image_style'],
      '#states' => [
        'visible' => [
          '.image_style_options' => ['value' => 'image_style'],
        ],
      ],
    ];

    $form['media_settings']['different_image_style'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use different styles for posts'),
      '#default_value' => $this->configuration['different_image_style'],
      '#attributes' => ['class' => ['different_image_style']],
      '#ajax' => [
        'callback' => [$this, 'formSettingsAjax'],
        'event' => 'change',
        'wrapper' => 'instagram-form--settings',
        'progress' => [
          'type' => 'throbber',
        ],
      ],
      '#states' => [
        'visible' => [
          '.image_style_options' => ['value' => 'image_style'],
        ],
      ],
    ];

    // IMAGE STYLE
    $form['image_styles_rendering'] = [
      '#type' => 'details',
      '#title' => $this->t('Image styles'),
      '#description_display' => 'none',
      '#attributes' => [
        'open' => TRUE,
      ],
      '#states' => [
        'visible' => [
          '.image_style_options' => ['value' => 'image_style'],
          'and',
          '.different_image_style' => ['checked' => TRUE],
        ],
      ],
      'styles' => $this->generateImageStyleSelect($this->configuration['limit'],
        []),
    ];

    // RESPONSIVE
    if ($responsive_style_access) {
      $form['media_settings']['responsive_image_style'] = [
        '#type' => 'select',
        '#title' => $this->t('Responsive image style'),
        '#options' => $this->getResponsiveStylesImage(),
        '#default_value' => $config['responsive_image_style'] ?? '',
        '#attributes' => ['class' => ['responsive_image_style']],
        '#states' => [
          'visible' => [
            '.image_style_options' => ['value' => 'responsive_image_style'],
          ],
        ],
      ];

      // SELECTS
      $form['responsive_image_rendering'] = [
        '#type' => 'details',
        '#title' => $this->t('Responsive styles'),
        '#attributes' => [
          'open' => TRUE,
        ],
        '#states' => [
          'visible' => [
            '.image_style_options' => ['value' => 'responsive_image_style'],
            'and',
            '.different_responsive_style' => ['checked' => TRUE],
          ],
        ],
        'styles' => $this->generateResponsiveStyleSelect($this->configuration['limit']),
      ];

      // CHECKBOX
      $form['media_settings']['different_responsive_style'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Use different responsive styles for posts'),
        '#default_value' => $this->configuration['different_responsive_style'] ?? FALSE,
        '#attributes' => ['class' => ['different_responsive_style']],
        '#ajax' => [
          'callback' => [$this, 'formSettingsAjax'],
          'event' => 'change',
          'wrapper' => 'instagram-form--settings',
          'progress' => [
            'type' => 'throbber',
          ],
        ],
        '#states' => [
          'visible' => [
            '.image_style_options' => ['value' => 'responsive_image_style'],
          ],
        ],
      ];
    }

    $form['media_settings']['limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Number of images to display'),
      '#default_value' => $config['limit'] ?? 6,
      '#attributes' => [
        'min' => '1',
      ],
      '#ajax' => [
        'callback' => [$this, 'formSettingsAjax'],
        'event' => 'change',
        'wrapper' => 'instagram-form--settings',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Build image styles count ...'),
        ],
      ],
    ];

    $form['media_settings']['video'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Hide videos'),
      '#required' => FALSE,
      '#default_value' => $config['video'] ?? FALSE,
      '#attributes' => ['class' => ['instagram_media_video']],
    ];

    $form['media_settings']['autoplay'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Autoplay Instagram Videos'),
      '#required' => FALSE,
      '#default_value' => $config['autoplay'] ?? FALSE,
      '#states' => [
        'visible' => [
          '.instagram_media_video' => ['checked' => FALSE],
        ],
      ],
    ];

    // Post settings

    $form['post_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Post settings'),
    ];

    $form['post_settings']['post_caption'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display Instagram post caption'),
      '#default_value' => $config['post_caption'] ?? FALSE,
    ];

    $form['post_settings']['insights'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('View insights'),
      '#default_value' => $config['insights'] ?? FALSE,
    ];

    $form['post_settings']['links'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('View links'),
      '#default_value' => $config['links'] ?? FALSE,
    ];

    $header_mode_options = [
      'only_link' => $this->t('Only Link'),
      'tag_and_link' => $this->t('Tag and Link'),
      'all_info' => $this->t('Looks like Instagram'),
    ];

    $form['post_settings']['header_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Header Mode'),
      '#options' => $header_mode_options,
      '#default_value' => $config['header_mode'] ?: 'tag_and_link',
      '#states' => [
        'visible' => [
          ':input[name="settings[post_settings][links]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Styles settings

    $form['styling_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Styling settings'),
    ];

    $form['styling_settings']['swiper'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Has swiper'),
      '#default_value' => $config['swiper'] ?? FALSE,
      '#attributes' => ['class' => ['instagram_media_video']],

    ];

    $form['styling_settings']['swiper_arrow'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Swiper arrow'),
      '#default_value' => $config['swiper_arrow'] ?? FALSE,
      '#states' => [
        'visible' => [
          '.instagram_media_video' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['styling_settings']['swiper_pagination'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Swiper pagination'),
      '#default_value' => $config['swiper_pagination'] ?? FALSE,
      '#states' => [
        'visible' => [
          '.instagram_media_video' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['styling_settings']['fancybox'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Fancybox'),
      '#default_value' => $config['fancybox'] ?? FALSE,
    ];

    $form['styling_settings']['disabled_style'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Disabled css styles'),
      '#required' => FALSE,
      '#default_value' => $config['disabled_style'] ?: '',
      '#description' => $this->t('Disables all styling to overwrite custom styles'),
      '#attributes' => ['class' => ['disabled_style']],
    ];

    $grid_options = [
      'no-grid' => $this->t('No grid'),
      'grid-4' => $this->t('Grid 4'),
      'grid-2-4' => $this->t('Grid 2-4'),
      'grid-5' => $this->t('Grid 5'),
      'grid-2-5' => $this->t('Grid 2-5'),
      'grid-6' => $this->t('Grid 6'),
      'grid-2-6' => $this->t('Grid 2-6'),
    ];

    $form['styling_settings']['grid'] = [
      '#type' => 'select',
      '#title' => $this->t('Grid settings'),
      '#options' => $grid_options,
      '#description' => 'The grid option, from the mobile version to the PC. That is, 2 in mobile, and 6 in PC',
      '#required' => FALSE,
      '#default_value' => $config['grid'] ?: 'no-grid',
      '#states' => [
        'invisible' => [
          '.disabled_style' => ['checked' => TRUE],
        ],
      ],
    ];

    if ($form_state instanceof SubformStateInterface) {
      $form_state = $form_state->getCompleteFormState();
      $values = $form_state->getValues();

      if (!empty($values['settings'])) {
        // IMAGE STYLES

        $form['image_styles_rendering']['styles'] = $this->generateImageStyleSelect($values['settings']['media_settings']['limit'],
          $values['settings']);

        // RESPONSIVE IMAGE STYLES

        if ($responsive_style_access) {
          $form['responsive_image_rendering']['styles'] = $this->generateResponsiveStyleSelect($values['settings']['media_settings']['limit'],
            $values['settings']);
        }
      }
    }
    $form_state->setRebuild();
    return $form;
  }

  /**
   * Return renderable array of image styles
   *
   * @param integer $count
   * @param array $values
   *
   * @return array
   */

  public function generateImageStyleSelect(int $count, array $values): array {
    $styles = image_style_options();
    $container = [];

    for ($i = 0; $i < $count; $i++) {
      if (empty($values)) {
        $value = @$this->configuration['image_styles_rendering']["image_style_$i"];
      }
      else {
        $value = $values['image_styles_rendering']['styles']["image_style_$i"];
      }

      $container["image_style_$i"] = [
        '#type' => 'select',
        '#title' => $this->t('Image style for post @count', ['@count' => $i]),
        '#default_value' => !empty($value) ? $value : NULL,
        '#options' => ['' => $this->t('- None -')] + $styles,
      ];
    }

    return $container;
  }

  /**
   * Build responsive_image_style options for respective
   * field settings.
   *
   * @return array
   *  Array of options for select type.
   */
  public function getResponsiveStylesImage(): array {
    $site_styles = $this->entityTypeManager->getStorage('responsive_image_style')
      ->loadMultiple();
    $image_style = [];
    if (!empty($site_styles)) {
      $image_style[''] = t('- None -');
    }
    foreach ($site_styles as $key => $style) {
      $image_style[$key] = $key;
    }
    return $image_style;
  }

  /**
   * Return renderable array of image styles
   *
   * @param integer $count
   * @param array $values
   *
   * @return array
   */

  public function generateResponsiveStyleSelect(
    int $count,
    array $values = []
  ): array {
    $styles = $this->getResponsiveStylesImage();
    $container = [];

    for ($i = 0; $i < $count; $i++) {
      $value = NULL;

      if (empty($values)) {
        $value = @$this->configuration['responsive_image_rendering']["responsive_style_$i"];
      }
      else {
        $value = $values['responsive_image_rendering']['styles']["responsive_style_$i"];
      }
      $container["responsive_style_$i"] = [
        '#type' => 'select',
        '#title' => $this->t('Responsive image style for post @count',
          ['@count' => $i]),
        '#default_value' => !empty($value) ? $value : NULL,
        '#options' => $styles,
      ];
    }

    return $container;
  }

  /**
   * Form ajax callback.
   *
   * @param array $form
   *
   * @return array
   */

  public function formSettingsAjax(
    array $form
  ): array {
    return $form['settings'];
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function blockSubmit($form, FormStateInterface $form_state): void {
    $this->configuration['token'] = $form_state->getValue('token');
    $this->configuration['image_style_options'] = $form_state->getValue('media_settings')['image_style_options'];
    $this->configuration['image_style'] = $form_state->getValue('media_settings')['image_style'];
    $this->configuration['responsive_image_style'] = $form_state->getValue('media_settings')['responsive_image_style'];
    $this->configuration['limit'] = $form_state->getValue('media_settings')['limit'];
    $this->configuration['video'] = $form_state->getValue('media_settings')['video'];
    $this->configuration['autoplay'] = $form_state->getValue('media_settings')['autoplay'];
    $this->configuration['post_caption'] = $form_state->getValue('post_settings')['post_caption'];
    $this->configuration['insights'] = $form_state->getValue('post_settings')['insights'];
    $this->configuration['links'] = $form_state->getValue('post_settings')['links'];
    $this->configuration['header_mode'] = $form_state->getValue('post_settings')['header_mode'];
    $this->configuration['swiper'] = $form_state->getValue('styling_settings')['swiper'];
    $this->configuration['swiper_arrow'] = $form_state->getValue('styling_settings')['swiper_arrow'];
    $this->configuration['swiper_pagination'] = $form_state->getValue('styling_settings')['swiper_pagination'];
    $this->configuration['fancybox'] = $form_state->getValue('styling_settings')['fancybox'];
    $this->configuration['disabled_style'] = $form_state->getValue('styling_settings')['disabled_style'];
    $this->configuration['grid'] = $form_state->getValue('styling_settings')['grid'];
    $buildInfo = $form_state->getBuildInfo();

    $values = $form_state->getValues();
    $responsive_style_access = FALSE;

    if ($this->moduleHandler->moduleExists('responsive_image')) {
      $responsive_style_access = TRUE;
    }
    // Preset
    $this->configuration['different_image_style'] = $values['media_settings']['different_image_style'];
    if ($responsive_style_access) {
      $this->configuration['different_responsive_style'] = $values['media_settings']['different_responsive_style'];
    }

    // Image styles
    $this->configuration['image_styles_rendering'] = [];
    if ($this->configuration['different_image_style']) {
      $this->configuration['image_styles_rendering'] = $values['image_styles_rendering']['styles'];
    }

    // Responsive styles
    $this->configuration['responsive_image_rendering'] = [];
    if ($responsive_style_access && $this->configuration['different_responsive_style']) {
      $this->configuration['responsive_image_rendering'] = $values['responsive_image_rendering']['styles'];
    }

    $instagram_media_path = 'public://instagram_media/';

    if (isset($buildInfo['form_id']) && $buildInfo['form_id'] === 'block_form') {
      $blockId = $buildInfo['callback_object']->getEntity()->id();
      $instagram_media_path .= $blockId . '/';
      $this->configuration['block_id'] = $blockId;
    }
    $this->configuration['media_path'] = $instagram_media_path;

    $this->instagramMediaService->submitInstagramMediaData($this->configuration['token'],
      $this->configuration['limit'],
      $this->configuration['block_id'],
      $this->configuration['post_caption'],
      $this->configuration['insights'],
      $this->configuration['links'],
      $this->configuration['media_path'],
    );
    parent::blockSubmit($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    return AccessResult::allowedIfHasPermission($account, 'access content');
  }

}
